@@ -390,7 +390,55 @@ class MonkAIRunHooks(RunHooks):
         1. Direct internal tool items: item.type == 'web_search_call'
         2. Wrapped in ToolCallItem: item.type == 'tool_call_item' with item.raw_item.type == 'web_search_call'
         3. web_searches array on output object
+        4. new_items array on output (RunResult)
+        5. Nested in output.output for streaming results
         """
+        # DEBUG: Detailed logging to investigate structure
+        print(f"\n[MonkAI DEBUG] ========== _capture_internal_tools START ==========")
+        print(f"[MonkAI DEBUG] agent_name: {agent_name}")
+        print(f"[MonkAI DEBUG] output type: {type(output)}")
+        print(f"[MonkAI DEBUG] output class name: {output.__class__.__name__ if output else 'None'}")
+        
+        # Log all attributes of output
+        if output:
+            output_attrs = [attr for attr in dir(output) if not attr.startswith('_')]
+            print(f"[MonkAI DEBUG] output public attrs: {output_attrs}")
+            
+            # Log specific interesting attributes
+            for attr in ['raw_items', 'new_items', 'items', 'output', 'final_output', 'messages', 'raw_response', 'data']:
+                if hasattr(output, attr):
+                    val = getattr(output, attr, None)
+                    val_type = type(val).__name__ if val is not None else 'None'
+                    val_len = len(val) if hasattr(val, '__len__') and not isinstance(val, str) else 'N/A'
+                    print(f"[MonkAI DEBUG] output.{attr}: type={val_type}, len={val_len}")
+                    # If it's a list, log first few items
+                    if isinstance(val, (list, tuple)) and len(val) > 0:
+                        for i, item in enumerate(val[:5]):
+                            item_type = getattr(item, 'type', 'no_type_attr')
+                            item_class = item.__class__.__name__
+                            print(f"[MonkAI DEBUG]   output.{attr}[{i}]: class={item_class}, type={item_type}")
+                            # Check for raw_item nested structure
+                            if hasattr(item, 'raw_item'):
+                                raw_item = item.raw_item
+                                raw_type = self._get_attr(raw_item, 'type')
+                                raw_class = raw_item.__class__.__name__ if raw_item else 'None'
+                                print(f"[MonkAI DEBUG]     raw_item: class={raw_class}, type={raw_type}")
+        
+        # Log context attributes
+        if context:
+            print(f"[MonkAI DEBUG] context type: {type(context)}")
+            context_attrs = [attr for attr in dir(context) if not attr.startswith('_')]
+            print(f"[MonkAI DEBUG] context public attrs (first 20): {context_attrs[:20]}")
+            
+            if hasattr(context, 'response'):
+                resp = context.response
+                if resp:
+                    print(f"[MonkAI DEBUG] context.response type: {type(resp)}")
+                    if hasattr(resp, 'raw_items'):
+                        print(f"[MonkAI DEBUG] context.response.raw_items exists, len={len(resp.raw_items) if resp.raw_items else 0}")
+        
+        print(f"[MonkAI DEBUG] ========== END DEBUG INFO ==========\n")
+        
         # Map of internal tool types
         internal_tool_types = {
             'web_search_call': 'web_search',
@@ -400,26 +448,55 @@ class MonkAIRunHooks(RunHooks):
         }
         
         raw_items = None
+        source = None
         
-        # Try to get raw_items from output
+        # Try to get raw_items from various locations
+        # Location 1: output.raw_items
         if hasattr(output, 'raw_items') and output.raw_items:
             raw_items = output.raw_items
-        # Try from context.response
-        elif hasattr(context, 'response') and hasattr(context.response, 'raw_items'):
+            source = 'output.raw_items'
+        # Location 2: output.new_items (RunResult structure)
+        elif hasattr(output, 'new_items') and output.new_items:
+            raw_items = output.new_items
+            source = 'output.new_items'
+        # Location 3: output.items
+        elif hasattr(output, 'items') and output.items:
+            raw_items = output.items
+            source = 'output.items'
+        # Location 4: context.response.raw_items
+        elif hasattr(context, 'response') and context.response and hasattr(context.response, 'raw_items') and context.response.raw_items:
             raw_items = context.response.raw_items
-        # Try from output as iterable
+            source = 'context.response.raw_items'
+        # Location 5: output.output (nested for streaming)
+        elif hasattr(output, 'output') and output.output:
+            nested = output.output
+            if hasattr(nested, 'raw_items') and nested.raw_items:
+                raw_items = nested.raw_items
+                source = 'output.output.raw_items'
+            elif hasattr(nested, 'new_items') and nested.new_items:
+                raw_items = nested.new_items
+                source = 'output.output.new_items'
+        # Location 6: output as iterable (last resort)
         elif hasattr(output, '__iter__') and not isinstance(output, str):
             try:
                 raw_items = list(output)
+                source = 'output (iterable)'
             except:
                 pass
         
+        if raw_items:
+            print(f"[MonkAI DEBUG] Found raw_items from {source}, count={len(raw_items)}")
+        else:
+            print(f"[MonkAI DEBUG] No raw_items found in any location")
+        
         captured_count = 0
         
         # Process raw_items if found
         if raw_items:
-            for item in raw_items:
+            for idx, item in enumerate(raw_items):
                 item_type = getattr(item, 'type', None)
+                item_class = item.__class__.__name__
+                print(f"[MonkAI DEBUG] Processing item[{idx}]: class={item_class}, type={item_type}")
                 
                 # Case 1: Direct internal tool type (item.type == 'web_search_call')
                 if item_type in internal_tool_types:
@@ -427,6 +504,7 @@ class MonkAIRunHooks(RunHooks):
                     tool_details = self._parse_internal_tool_details(item, item_type)
                     self._add_internal_tool_message(agent_name, item, item_type, tool_name, tool_details)
                     captured_count += 1
+                    print(f"[MonkAI DEBUG] Captured direct internal tool: {tool_name}")
                 
                 # Case 2: Wrapped in tool_call_item (item.type == 'tool_call_item')
                 elif item_type == 'tool_call_item':
@@ -434,24 +512,59 @@ class MonkAIRunHooks(RunHooks):
                     if raw_item:
                         # Get the actual type from raw_item (can be object or dict)
                         actual_type = self._get_attr(raw_item, 'type')
+                        print(f"[MonkAI DEBUG] tool_call_item raw_item.type={actual_type}")
                         
                         if actual_type in internal_tool_types:
                             tool_name = internal_tool_types[actual_type]
                             tool_details = self._parse_internal_tool_details(raw_item, actual_type)
                             self._add_internal_tool_message(agent_name, raw_item, actual_type, tool_name, tool_details)
                             captured_count += 1
+                            print(f"[MonkAI DEBUG] Captured wrapped internal tool: {tool_name}")
+                
+                # Case 3: Check if item itself has nested structure we haven't checked
+                else:
+                    # Check for any nested items that might contain internal tools
+                    for nested_attr in ['raw_item', 'item', 'data', 'content']:
+                        if hasattr(item, nested_attr):
+                            nested = getattr(item, nested_attr)
+                            if nested:
+                                nested_type = self._get_attr(nested, 'type')
+                                if nested_type in internal_tool_types:
+                                    tool_name = internal_tool_types[nested_type]
+                                    tool_details = self._parse_internal_tool_details(nested, nested_type)
+                                    self._add_internal_tool_message(agent_name, nested, nested_type, tool_name, tool_details)
+                                    captured_count += 1
+                                    print(f"[MonkAI DEBUG] Captured nested ({nested_attr}) internal tool: {tool_name}")
         
-        # Case 3: Check for web_searches array directly on output (fallback)
+        # Case 4: Check for web_searches array directly on output (fallback)
         if hasattr(output, 'web_searches') and output.web_searches:
+            print(f"[MonkAI DEBUG] Found output.web_searches, count={len(output.web_searches)}")
             for ws in output.web_searches:
                 ws_type = self._get_attr(ws, 'type')
                 if ws_type == 'web_search_call':
                     tool_details = self._parse_internal_tool_details(ws, 'web_search_call')
                     self._add_internal_tool_message(agent_name, ws, 'web_search_call', 'web_search', tool_details)
                     captured_count += 1
+                    print(f"[MonkAI DEBUG] Captured web_search from web_searches array")
+        
+        # Case 5: Check output.data for RunResult with nested data
+        if hasattr(output, 'data') and output.data:
+            data = output.data
+            if hasattr(data, 'raw_items') and data.raw_items:
+                print(f"[MonkAI DEBUG] Found output.data.raw_items, processing...")
+                for item in data.raw_items:
+                    item_type = getattr(item, 'type', None)
+                    if item_type in internal_tool_types:
+                        tool_name = internal_tool_types[item_type]
+                        tool_details = self._parse_internal_tool_details(item, item_type)
+                        self._add_internal_tool_message(agent_name, item, item_type, tool_name, tool_details)
+                        captured_count += 1
+                        print(f"[MonkAI DEBUG] Captured from output.data.raw_items: {tool_name}")
         
         if captured_count > 0:
             print(f"[MonkAI] Captured {captured_count} internal tool(s)")
+        else:
+            print(f"[MonkAI DEBUG] No internal tools captured - may need to check different structure")
     
     def _get_attr(self, obj: Any, attr: str, default: Any = None) -> Any:
         """Get attribute from object or dict safely"""
@@ -485,12 +598,42 @@ class MonkAIRunHooks(RunHooks):
         
         if item_type == 'web_search_call':
             action = self._get_attr(item, 'action')
+            result = self._get_attr(item, 'result')
+            
+            # Debug logging to understand structure
+            print(f"[MonkAI DEBUG] web_search_call - action type: {type(action)}")
+            print(f"[MonkAI DEBUG] web_search_call - result type: {type(result)}")
+            if result:
+                if hasattr(result, '__dict__'):
+                    result_attrs = [attr for attr in dir(result) if not attr.startswith('_')]
+                    print(f"[MonkAI DEBUG] web_search_call - result attrs: {result_attrs[:10]}")
+                elif isinstance(result, dict):
+                    print(f"[MonkAI DEBUG] web_search_call - result keys: {list(result.keys())[:10]}")
+                elif isinstance(result, list):
+                    print(f"[MonkAI DEBUG] web_search_call - result is list with {len(result)} items")
+                    if result:
+                        first_item = result[0]
+                        print(f"[MonkAI DEBUG] web_search_call - first item type: {type(first_item)}")
+            
+            # Extract sources from result object (not from action!)
+            sources = None
+            if result:
+                # Try various attribute names where sources might be stored
+                sources = (
+                    self._get_attr(result, 'sources') or 
+                    self._get_attr(result, 'results') or
+                    self._get_attr(result, 'web_results')
+                )
+                # If sources is still None but result is a list, use result directly as sources
+                if sources is None and isinstance(result, list):
+                    sources = result
+            
             return {
                 "arguments": {
                     "query": self._get_attr(action, 'query') if action else None,
-                    "sources": self._get_attr(action, 'sources') if action else None,
+                    "sources": sources,  # From result, not action
                 },
-                "result": self._get_attr(item, 'result')
+                "result": result  # Full result object for detailed view
             }
         
         elif item_type == 'file_search_call':
@@ -543,18 +686,125 @@ class MonkAIRunHooks(RunHooks):
             except:
                 pass
     
-    @staticmethod
-    def run_with_tracking(agent: Agent, user_input: str, hooks: 'MonkAIRunHooks', **kwargs):
+    def _capture_internal_tools_from_result(self, result: Any, agent_name: str) -> None:
         """
-        Convenience wrapper for Runner.run() that automatically captures user input.
+        Capture internal tools from the complete RunResult object.
+        Called AFTER Runner.run() returns, when we have access to new_items and raw_responses.
+        
+        This is the CORRECT place to capture internal tools because:
+        - on_agent_end only receives final_output (a string), not the full RunResult
+        - RunResult.new_items contains all ToolCallItem objects with web_search_call, etc.
+        """
+        print(f"\n[MonkAI DEBUG] ========== _capture_internal_tools_from_result START ==========")
+        print(f"[MonkAI DEBUG] result type: {type(result)}")
+        print(f"[MonkAI DEBUG] result class: {result.__class__.__name__ if result else 'None'}")
+        
+        internal_tool_types = {
+            'web_search_call': 'web_search',
+            'file_search_call': 'file_search',
+            'code_interpreter_call': 'code_interpreter',
+            'computer_call': 'computer_use',
+        }
+        
+        captured_count = 0
+        
+        # Check for new_items (primary source)
+        new_items = getattr(result, 'new_items', None)
+        if new_items:
+            print(f"[MonkAI DEBUG] Found result.new_items with {len(new_items)} items")
+            captured_count += self._process_items_for_internal_tools(new_items, agent_name, 'result.new_items', internal_tool_types)
+        else:
+            print(f"[MonkAI DEBUG] No new_items found in result")
+        
+        # Also check raw_responses as backup
+        raw_responses = getattr(result, 'raw_responses', None)
+        if raw_responses:
+            print(f"[MonkAI DEBUG] Found result.raw_responses with {len(raw_responses)} items")
+            for i, resp in enumerate(raw_responses):
+                resp_output = getattr(resp, 'output', None)
+                if resp_output and isinstance(resp_output, list):
+                    captured_count += self._process_items_for_internal_tools(resp_output, agent_name, f'result.raw_responses[{i}].output', internal_tool_types)
+        
+        if captured_count > 0:
+            print(f"[MonkAI] Captured {captured_count} internal tool(s) from RunResult")
+            
+            # Add internal tools to the LAST buffered record (the one from on_agent_end)
+            if self._batch_buffer:
+                last_record = self._batch_buffer[-1]
+                # Append the internal tool messages to the record's messages
+                # The _messages list now contains the internal tools we just captured
+                internal_tool_messages = [m for m in self._messages if getattr(m, 'is_internal_tool', False)]
+                if internal_tool_messages:
+                    # Update the last record's msg list
+                    existing_msgs = last_record.msg if isinstance(last_record.msg, list) else []
+                    last_record.msg = existing_msgs + internal_tool_messages
+                    print(f"[MonkAI DEBUG] Added {len(internal_tool_messages)} internal tool messages to buffered record")
+        
+        print(f"[MonkAI DEBUG] ========== _capture_internal_tools_from_result END ==========\n")
+    
+    def _process_items_for_internal_tools(self, items: list, agent_name: str, source: str, internal_tool_types: dict) -> int:
+        """Process a list of items to extract internal tools. Returns count of captured tools."""
+        captured_count = 0
+        
+        for i, item in enumerate(items):
+            # Get item type - handle both objects and dicts
+            item_type = self._get_attr(item, 'type')
+            item_class = type(item).__name__
+            
+            print(f"[MonkAI DEBUG] {source}[{i}]: class={item_class}, type={item_type}")
+            
+            # Case 1: Direct internal tool (item.type == 'web_search_call')
+            if item_type in internal_tool_types:
+                tool_name = internal_tool_types[item_type]
+                tool_details = self._parse_internal_tool_details(item, item_type)
+                self._add_internal_tool_message(agent_name, item, item_type, tool_name, tool_details)
+                captured_count += 1
+                print(f"[MonkAI DEBUG] ✅ Found direct internal tool: {tool_name}")
+                continue
+            
+            # Case 2: Wrapped in ToolCallItem (item.type == 'tool_call_item')
+            if item_type == 'tool_call_item':
+                raw_item = self._get_attr(item, 'raw_item')
+                if raw_item:
+                    actual_type = self._get_attr(raw_item, 'type')
+                    print(f"[MonkAI DEBUG] tool_call_item has raw_item.type={actual_type}")
+                    if actual_type in internal_tool_types:
+                        tool_name = internal_tool_types[actual_type]
+                        tool_details = self._parse_internal_tool_details(raw_item, actual_type)
+                        self._add_internal_tool_message(agent_name, raw_item, actual_type, tool_name, tool_details)
+                        captured_count += 1
+                        print(f"[MonkAI DEBUG] ✅ Found wrapped internal tool: {tool_name}")
+        
+        return captured_count
+    
+    @staticmethod
+    async def run_with_tracking(agent: Agent, user_input: str, hooks: 'MonkAIRunHooks', **kwargs):
+        """
+        Convenience wrapper for Runner.run() that automatically captures user input
+        and internal tools from the complete RunResult.
+        
         This is the recommended way to use MonkAIRunHooks.
         
+        BREAKING CHANGE in v0.2.4: This method is now async and must be awaited.
+        
         Usage:
             hooks = MonkAIRunHooks(...)
             result = await MonkAIRunHooks.run_with_tracking(agent, "Hello", hooks)
         """
         # Set user input before running
         hooks.set_user_input(user_input)
+        
         # Import Runner here to avoid circular dependency
         from agents import Runner
-        return Runner.run(agent, user_input, hooks=hooks, **kwargs)
+        
+        # Run the agent and get complete RunResult
+        result = await Runner.run(agent, user_input, hooks=hooks, **kwargs)
+        
+        # Capture internal tools from the COMPLETE RunResult (has new_items, raw_responses)
+        hooks._capture_internal_tools_from_result(result, agent.name)
+        
+        # Force flush any remaining buffered records that now include the internal tools
+        if hooks._batch_buffer:
+            await hooks._flush_batch()
+        
+        return result
